/*
 * Copyright 2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.engine.servlet;

import java.security.Principal;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.jetspeed.engine.servlet.ServletRequestImpl;
import org.apache.jetspeed.request.JetspeedRequestContext;
import org.apache.jetspeed.services.JetspeedSecurity;
import org.apache.jetspeed.services.rundata.JetspeedRunDataService;
import org.apache.jetspeed.services.security.JetspeedSecurityException;
import org.apache.pluto.om.common.SecurityRole;
import org.apache.pluto.om.common.SecurityRoleRef;
import org.apache.pluto.om.common.SecurityRoleRefSet;
import org.apache.pluto.om.common.SecurityRoleSet;
import org.apache.pluto.om.portlet.PortletDefinition;
import org.apache.pluto.om.window.PortletWindow;
import org.apache.turbine.services.TurbineServices;
import org.apache.turbine.services.rundata.RunDataService;

/**
 * @version $Id: FusionServletRequestImpl.java,v 1.5 2004/09/01 21:21:56 taylor Exp $
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @author <a href="mailto:jford@apache.org">Jeremy Ford</a>
 */
public class FusionServletRequestImpl extends ServletRequestImpl
{
    /** Logger */
    private static final Log log = LogFactory.getLog(FusionServletRequestImpl.class);
    
    private PortletWindow portletWindow;

    public FusionServletRequestImpl(javax.servlet.http.HttpServletRequest servletRequest, PortletWindow window)
    {
        super(servletRequest, window);
        
        this.portletWindow = window;
    }
    
    private String getUserName()
    {
        JetspeedRunDataService runDataService =
            (JetspeedRunDataService)TurbineServices.getInstance()
                .getService(RunDataService.SERVICE_NAME);
        return runDataService.getCurrentRunData().getJetspeedUser().getUserName();        
    }
    
    /**
     * @see javax.servlet.http.HttpServletRequest#isUserInRole(java.lang.String)
     */
    public boolean isUserInRole(String roleName)
    {
        boolean result = false;
        
        if (roleName != null && roleName.length() > 0)
        {
            try
            {
                String mappedName = mapRoleRefToRole(roleName);
                result = JetspeedSecurity.hasRole(getUserName(), mappedName);
            }
            catch (JetspeedSecurityException e)
            {
                log.error("Failed to find role " + roleName);
            }
                        
            if(!result)
            {
                result = super.isUserInRole(roleName);
            }
        }
        
        return result;
    }
    
    private String mapRoleRefToRole(String roleName)
    {
        PortletDefinition portletDefinition = portletWindow.getPortletEntity().getPortletDefinition();
        SecurityRoleRefSet roleRefSet = portletDefinition.getInitSecurityRoleRefSet();
        SecurityRoleSet roleSet = portletDefinition.getPortletApplicationDefinition().getWebApplicationDefinition()
                .getSecurityRoles();

        Iterator roleRefIter = roleRefSet.iterator();
        while (roleRefIter.hasNext())
        {
            SecurityRoleRef roleRef = (SecurityRoleRef) roleRefIter.next();
            if (roleName.equals(roleRef.getRoleName()))
            {
                String roleLinkName = roleRef.getRoleLink();
                if (roleLinkName == null || roleLinkName.length() == 0)
                {
                    roleLinkName = roleName;
                }
                Iterator roleIter = roleSet.iterator();
                while (roleIter.hasNext())
                {
                    SecurityRole role = (SecurityRole) roleIter.next();
                    if (roleLinkName.equals(role.getRoleName()))
                        return role.getRoleName();
                }
            }
        }
        return roleName;        
    }
    
    public void setAttribute( String name, Object value )
    {
        this._getHttpServletRequest().setAttribute(name, value);        
    }
    
    public Principal getUserPrincipal()
    {
        JetspeedRequestContext context = (JetspeedRequestContext) getAttribute("org.apache.jetspeed.request.RequestContext");
        if (context != null)
        {
            Set principals = context.getSubject().getPrincipals();
            if (principals != null)
            {
                Iterator it = principals.iterator();
                if (it.hasNext())
                {
                    return (Principal) it.next();
                }
            }
        }
        return super.getUserPrincipal();

    }
    
    
}
