/*
 * Copyright 2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.modules.actions;

import java.util.Iterator;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.jetspeed.Jetspeed;
import org.apache.jetspeed.PortalReservedParameters;
import org.apache.jetspeed.container.state.MutableNavigationalState;
import org.apache.jetspeed.container.state.NavigationalState;
import org.apache.jetspeed.container.state.NavigationalStateComponent;
import org.apache.jetspeed.container.url.PortalURL;
import org.apache.jetspeed.engine.Engine;
import org.apache.jetspeed.modules.actions.JetspeedAccessController;
import org.apache.jetspeed.om.profile.Entry;
import org.apache.jetspeed.om.profile.Parameter;
import org.apache.jetspeed.om.profile.Portlets;
import org.apache.jetspeed.portal.Portlet;
import org.apache.jetspeed.portal.PortletException;
import org.apache.jetspeed.portal.PortletState;
import org.apache.jetspeed.request.RequestContext;
import org.apache.jetspeed.request.RequestContextComponent;
import org.apache.jetspeed.services.PortletFactory;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.resources.FusionResources;
import org.apache.jetspeed.services.resources.JetspeedResources;
import org.apache.jetspeed.services.rundata.JetspeedRunData;
import org.apache.pluto.om.window.PortletWindow;
import org.apache.turbine.TurbineConstants;
import org.apache.turbine.util.RunData;

/**
 * <p>FusionAccessController serves two purposes:
 * <ul>
 * <li>Process the JSR 168 action phase. A portlet's action is executed here.</li>
 * <li>Delegate to standard Jetpeed-1 access controller processing.</li>
 * </ul>   
 *
 * @author <a href="mailto:david@bluesunrise.com">David Sean Taylor</a>
 * @version $Id: FusionAccessController.java,v 1.7 2005/05/24 21:12:36 taylor Exp $
 */
public class FusionAccessController extends JetspeedAccessController
{
    private static final JetspeedLogger log = JetspeedLogFactoryService.getLogger(FusionAccessController.class.getName());
    private static final String J2_ENTITY = FusionResources.getString(FusionResources.J2_ENTITY); 
    
    /**
     * Execute the action.
     *
     * @param data Turbine information.
     * @exception Exception, a generic exception.
     */
    public void doPerform( RunData data ) throws Exception
    {
        boolean throwit = false;
        try 
        {
            super.doPerform(data);         
        }
        catch (Throwable other)
        {
            data.setScreenTemplate(JetspeedResources.getString(TurbineConstants.TEMPLATE_ERROR));
            String message = other.getMessage() != null ? other.getMessage() : other.toString();
            data.setMessage(message);
            data.setStackTrace(org.apache.turbine.util.StringUtils.stackTrace(other), other);
            return;
        }
        
        RequestContextComponent contextComponent = null;
        RequestContext context = null;
        
        try
        {            
            HttpServletRequest request = data.getRequest();
            HttpServletResponse response = data.getResponse();            
            NavigationalStateComponent nav = (NavigationalStateComponent)Jetspeed.getComponentManager().getComponent(NavigationalStateComponent.class);
            
            //
            // Handle Action Phase via Action Pipeline
            //
            
            // System.out.println("*** ACTION PARAM FOUND!!!!!!!!!!! ****");                
            ServletConfig config = data.getServletConfig();
            Engine engine = Jetspeed.getEngine();
            contextComponent = (RequestContextComponent)Jetspeed.getComponentManager().getComponent(RequestContextComponent.class);
            context = contextComponent.create(request, response, config);

            
            //
            // Sync up navigational state for J1 Portlet Control decorators
            //            
            //if (state.)
            //String state[] = nav.parsePortalParameter(request, "_ns"); // TODO don't hard code
            //if (state != null)
            context.setAttribute(PortalReservedParameters.PIPELINE, PortalReservedParameters.ACTION_PIPELINE);                                
            engine.service(context);
            
            PortalURL url = context.getPortalURL();
            if (url != null)
            {
                NavigationalState state = url.getNavigationalState();                
                PortletWindow actionWindow = state.getPortletWindowOfAction();
                if (actionWindow == null)
                {
                    Iterator windows = state.getWindowIdIterator();
                    while (windows.hasNext())
                    {
                        String windowId = (String)windows.next();
                        PortletMode mode = state.getMode(windowId);
                        WindowState wstate = state.getState(windowId);
                        if (wstate != null || mode != null)
                        {
                            String windowState = "";
                            if (wstate != null)
                                windowState = wstate.toString();
                            
                            String portletMode = "";
                            if (mode != null)
                                portletMode = mode.toString();
                            
                            Portlets portlets = ((JetspeedRunData)data).getProfile().getDocument().getPortlets();
                            traverse(portlets, windowId, windowState, portletMode, (JetspeedRunData)data);                            
                        }
                    }
                    
                }
                if (actionWindow != null)
                {
                    String windowId = actionWindow.getId().toString();
                    //String windowId = nav.getWindowIdFromKey(state[0]);
                    //System.out.println("got token = " + state[0]+ ", window id = " + windowId + ", action = " + state[1]);
                    Portlets portlets = ((JetspeedRunData)data).getProfile().getDocument().getPortlets();
                    //traverse(portlets, windowId, state[1], (JetspeedRunData)data);
                    String windowState = state.getState(actionWindow).toString();
                    if (windowState == null)
                    {
                        windowState = "";
                    }
                    PortletMode mode = state.getMode(actionWindow);
                    String portletMode = "";
                    if (mode != null)
                        portletMode = mode.toString();
                    
                    traverse(portlets, windowId, windowState, portletMode, (JetspeedRunData)data);
                    throwit = true;
                }                
            }
            
        }
        catch (Throwable t)
        {
            t.printStackTrace();
            log.error("error in fusion access controller", t);
        }
        finally
        {
            if (contextComponent != null && context != null)
            {
                contextComponent.release(context);
            }            
            if (throwit)                
                throw new Exception("redirecting");
        }                
    }
    
    private boolean traverse(Portlets portlets, String windowId, String windowAction, String modeAction, JetspeedRunData jdata)
    {
        // First let's add all the Entries in the current level
        Iterator eItr = portlets.getEntriesIterator();
        while(eItr.hasNext())
        {
            Entry entry =  (Entry)eItr.next();
            //System.out.println("entry = " + entry.getId());
            Parameter param = entry.getParameter(J2_ENTITY);
            if (param == null)
            {
                continue;
            }
            //System.out.println("entity id = " + param.getValue());
            if (windowId.equals(param.getValue()))
            {
                //System.out.println("GOT A MATCH: " + entry.getId());
                if (windowAction.equals(WindowState.MAXIMIZED.toString()))
                {
                    jdata.getUser().setTemp("js_peid", entry.getId());
                }
                else if (windowAction.equals(WindowState.NORMAL.toString()))
                {
                    // restore
                    jdata.getUser().removeTemp("js_peid");
                }
                else if (windowAction.equals(WindowState.MINIMIZED.toString()))
                {
                    try
                    {
                        Portlet portlet = PortletFactory.getPortlet(entry);
                    
                        // Now set the portlet to minimized
                        if (( portlet != null )&&( portlet instanceof PortletState ))
                        {
                            ((PortletState)portlet).setMinimized( true, jdata );
                        }
                    }
                    catch (PortletException pe)
                    {
                        log.error(pe);
                    }
                    
                }
                                
                return true;
            }                                
            
        }
        //Now if there are child levels, drill down recursively
        if(portlets.getPortletsCount() > 0)
        {
            Iterator pItr = portlets.getPortletsIterator();
            while(pItr.hasNext())
            {
                Portlets childPortlets = (Portlets)pItr.next();
                if (traverse(childPortlets, windowId, windowAction, modeAction, jdata))
                {
                    return true;
                }
            }            
        }        
        return false;
        
    }
}
