package org.apache.jetspeed.fusion.modules.actions.portlets;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import org.apache.jetspeed.Jetspeed;
import org.apache.jetspeed.components.portletregistry.PortletRegistry;
import org.apache.jetspeed.modules.actions.portlets.CustomizeSetAction;
import org.apache.jetspeed.modules.actions.portlets.VelocityPortletAction;
import org.apache.jetspeed.om.common.portlet.PortletDefinitionComposite;
import org.apache.jetspeed.om.common.preference.PreferenceComposite;
import org.apache.jetspeed.om.registry.Parameter;
import org.apache.jetspeed.om.registry.PortletEntry;
import org.apache.jetspeed.portal.Portlet;
import org.apache.jetspeed.portal.PortletInstance;
import org.apache.jetspeed.portal.portlets.VelocityPortlet;
import org.apache.jetspeed.services.Registry;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.persistence.PersistenceManager;
import org.apache.jetspeed.services.resources.FusionResources;
import org.apache.jetspeed.services.rundata.JetspeedRunData;
import org.apache.pluto.om.common.PreferenceSet;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;


/**
 * @author Jeremy
 *
 * 
 */
public class CustomizeJSR168Action extends VelocityPortletAction
{
    /**
     * Static initialization of the logger for this class
     */    
    private static final JetspeedLogger logger = JetspeedLogFactoryService.getLogger(CustomizeJSR168Action.class.getName());
    
    private static final String PORTLET_APPLICATION_NAME = FusionResources.getString(FusionResources.PORTLET_APPLICATION_PARAMETER);
    private static final String PORTLET_DEFINITION_NAME = FusionResources.getString(FusionResources.PORTLET_DEFINITION_PARAMETER);
    
    /**
     * Subclasses must override this method to provide default behavior
     * for the portlet action
     *
     * <table>
     * <tr><th>Context      </th><th> Description</th></tr>
     * <!-- ---------  ------------------------- -->
     * <tr><td>action       </td><td> Action to use</td></tr>
     * <tr><td>current_skin </td><td> Current skin for this portlet INSTANCE</td></tr>
     * <tr><td>params       </td><td> List of configurable parameters from the REGISTRY entry.</td></tr>
     * <tr><td>portlet      </td><td> Portlet, not the Portlet Instance!</td></tr>
     * <tr><td>skins        </td><td> List of skins</td></tr>
     * <tr><td>security     </td><td> List of security ref</td></tr>
     * <tr><td>security_ref </td><td> Current securityRef for this portlet INSTANCE</td></tr>
     * </table>
     */
    protected void buildNormalContext( VelocityPortlet portlet,
                                       Context context,
                                       RunData rundata )
    {

        // generic context stuff
        context.put("skins", CustomizeSetAction.buildList(rundata, Registry.SKIN));
        context.put("securitys", CustomizeSetAction.buildList(rundata, Registry.SECURITY));

        // we should first retrieve the portlet to customize
        Portlet p = ((JetspeedRunData)rundata).getCustomized();
        
        PortletInstance instance = PersistenceManager.getInstance(p, rundata);
        context.put("portlet_instance", PersistenceManager.getInstance(p, rundata));

        if (p==null) return;

        // retrieve the portlet parameters
        PortletEntry entry = (PortletEntry)Registry.getEntry(Registry.PORTLET,p.getName());
        if(entry != null)
        {
            Parameter portletAppParam = entry.getParameter(PORTLET_APPLICATION_NAME);
            Parameter portletDefParam = entry.getParameter(PORTLET_DEFINITION_NAME);
            if(portletAppParam != null && portletDefParam != null)
            {
		        String portletApp = portletAppParam.getValue();
		        String portletDef = portletDefParam.getValue();
		        //String portletApp = PortletConfigState.getConfigParameter(p, PORTLET_APPLICATION_NAME, null);
		        //String portletDef = PortletConfigState.getConfigParameter(p, PORTLET_DEFINITION_NAME, null);
		        String uniqueId = portletApp + "::" + portletDef;
		        
		        logger.debug("Pref UniqueID: " + uniqueId);
		        System.out.println("Pref UniqueID: " + uniqueId);
		        
		        HashMap allPrefs = new HashMap();
		        
		        logger.info("MY ACTION FIRED!!!!");
		        System.out.println("MY ACTION FIRED!!!!");
		        
		        String entityId = "5"; //Preference Portlet
		        String userPrefsNode = "/user/" + rundata.getUser().getUserName() + "/portletprefs/" + entityId;
		        String entityPrefsNode = "/System/Jetspeed/entities/" + entityId;
		        
		        System.out.println("UserPrefNode: " + userPrefsNode);
		        System.out.println("EntityPrefNode: " + entityPrefsNode);
		        
		        //TODO:  get prefs from user and entity, add to hash
		        
		        PortletRegistry registry = (PortletRegistry) Jetspeed.getComponentManager().getComponent(PortletRegistry.class);
		        //PersistenceStore store = registry.getPersistenceStore();
		        
		        //store.getTransaction().begin();
		        
		        PortletDefinitionComposite pdef = registry.getPortletDefinitionByUniqueName(uniqueId);
		        PreferenceSet prefSet = pdef.getPreferenceSet();
		        Iterator prefIter = prefSet.iterator();
		        while (prefIter.hasNext())
		        {
		            PreferenceComposite pref = (PreferenceComposite) prefIter.next();
		            
		            String prefName = pref.getName();
		            String prefType = pref.getType();
		            
		            if(!allPrefs.containsKey(prefName))
		            {
		                String[] values = pref.getValueArray();
		                logger.debug("Found default pref " + prefName + " with type " + prefType + " with values " + values);
		                System.out.println("Found default pref " + prefName + " with type " + prefType + " with values " + values);
		                List valueList = arrayToList(values);
		                if(valueList != null)
		                {
		                    allPrefs.put(prefName, valueList);
		                }                
		            }
		        }
		        
		        //store.getTransaction().commit();
		        
		        context.put("preferences", allPrefs);
            }
        }
        
        /*
        PropertyManager pm = (PropertyManager) Jetspeed.getEngine().getComponentManager().getComponent(PropertyManager.class);
        
        String node = "/user/" + rundata.getUser().getUserName();
        Preferences userPrefs = Preferences.userRoot().node(node);
        
        try
        {
            String[] children = userPrefs.childrenNames();
            if(children != null)
            {
            	for(int i=0; i<children.length; i++)
            	{
            		String child = children[i];
            		Preferences childPref = userPrefs.node(child);
            	}
            }
        } catch (BackingStoreException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        */

        context.put("action", "portlets.CustomizeJSR168Action");
    }
    
    /**
     * This is untested!!!!
     * 
     * @param preferences
     * @param userNode
     */
    private void getUserPreferences(Map preferences, String userNode)
    {
        boolean nodeExists = false;
        try
        {
            nodeExists = Preferences.userRoot().nodeExists(userNode);
        }
        catch (BackingStoreException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        if(nodeExists)
        {
            Preferences userPrefs = Preferences.userRoot().node(userNode);
            String[] childNames = null;
            try
            {
                //will keys work just as well
                childNames = userPrefs.childrenNames();
            } catch (BackingStoreException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            if(childNames != null)
            {
                for(int i=0; i<childNames.length; i++)
                {
                    String childName = childNames[i];
                    String value = userPrefs.get(childName, null);
                    
                    ArrayList values = new ArrayList(1);
                    values.add(value);
                    preferences.put(childName, values);
                }
            }
        }
       
    }
    
    private List arrayToList(String[] array)
    {
        ArrayList list = null;
        
        if(array != null)
        {
            list = new ArrayList(array.length);
            for(int i=0; i<array.length; i++)
            {
                list.add(array[i]);
            }
        }
        
        return list;
    }
}
