/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.portal.portlets;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.ecs.ConcreteElement;
import org.apache.jetspeed.Jetspeed;
import org.apache.jetspeed.PortalReservedParameters;
import org.apache.jetspeed.components.portletentity.PortletEntityAccessComponent;
import org.apache.jetspeed.components.portletregistry.PortletRegistry;
import org.apache.jetspeed.container.state.NavigationalState;
import org.apache.jetspeed.container.state.MutableNavigationalState;
import org.apache.jetspeed.container.state.NavigationalStateComponent;
import org.apache.jetspeed.container.url.PortalURL;
import org.apache.jetspeed.container.window.PortletWindowAccessor;
import org.apache.jetspeed.engine.Engine;
import org.apache.jetspeed.portal.PortletException;
import org.apache.jetspeed.portal.PortletInstance;
import org.apache.jetspeed.portal.portlets.AbstractInstancePortlet;
import org.apache.jetspeed.request.RequestContext;
import org.apache.jetspeed.request.RequestContextComponent;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.resources.FusionResources;
import org.apache.jetspeed.services.rundata.JetspeedRunData;
import org.apache.jetspeed.util.JetspeedClearElement;
import org.apache.jetspeed.util.PortletSessionState;
import org.apache.pluto.om.entity.PortletEntity;
import org.apache.pluto.om.portlet.PortletDefinition;
import org.apache.pluto.om.window.PortletWindow;
import org.apache.turbine.util.RunData;

/**
 * JetspeedFusionPortlet
 *
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @version $Id: JetspeedFusionPortlet.java,v 1.12 2005/05/24 21:12:36 taylor Exp $
 */
public class JetspeedFusionPortlet extends AbstractInstancePortlet
{
    private static final JetspeedLogger log = JetspeedLogFactoryService.getLogger(JetspeedFusionPortlet.class.getName());    
    private static final String PORTLET_APPLICATION_NAME = FusionResources.getString(FusionResources.PORTLET_APPLICATION_PARAMETER);
    private static final String PORTLET_DEFINITION_NAME = FusionResources.getString(FusionResources.PORTLET_DEFINITION_PARAMETER);

    private static final String J2_ENTITY = FusionResources.getString(FusionResources.J2_ENTITY); 
    private static final String J1_HELP_ACTION = FusionResources.getString(FusionResources.J1_HELP_ACTION);
    private static final String J1_RESTORE_ACTION = FusionResources.getString(FusionResources.J1_RESTORE_ACTION);
    private static final String J1_MINIMIZE_ACTION = FusionResources.getString(FusionResources.J1_MINIMIZE_ACTION);
    private static final String J1_MAXIMIZE_ACTION = FusionResources.getString(FusionResources.J1_MAXIMIZE_ACTION);
    private static final String J1_EDIT_ACTION = FusionResources.getString(FusionResources.J1_EDIT_ACTION);
    private static final String J1_PRINT_ACTION = FusionResources.getString(FusionResources.J1_PRINT_ACTION);
        
    public static final String FUSION_NAV_STATE = "org.apache.jetspeed.fusion.navstate";
    
    private NavigationalStateComponent navState;
    private PortletWindowAccessor windowAccessor;
    private PortletEntityAccessComponent entityAccess;
    private PortletRegistry registryAccess;
        
    public void init()
    throws PortletException
    {
        navState = (NavigationalStateComponent) 
            Jetspeed.getComponentManager().getComponent(NavigationalStateComponent.class);
        windowAccessor = (PortletWindowAccessor) 
            Jetspeed.getComponentManager().getComponent(PortletWindowAccessor.class);
        entityAccess = (PortletEntityAccessComponent) 
            Jetspeed.getComponentManager().getComponent(PortletEntityAccessComponent.class);
        registryAccess = (PortletRegistry)
            Jetspeed.getComponentManager().getComponent(PortletRegistry.class);        
    }

    
    public ConcreteElement getContent(RunData data)
    {           
        JetspeedRunData jrun = (JetspeedRunData)data;
        HttpServletResponse response = data.getResponse();
        String entityId = null;
        
        try
        {
            PortletInstance instance = this.getInstance(jrun);
            entityId = instance.getAttribute(J2_ENTITY, "");
            String pa = getPortletConfig().getInitParameter(PORTLET_APPLICATION_NAME, null);
            String pd = getPortletConfig().getInitParameter(PORTLET_DEFINITION_NAME, null);
            String registryKey = pa + "::" + pd;
            
            PortletWindow window = getPortletWindow(entityId, registryKey); 
            if (window == null)
            {
                String msg = "Could not find portlet definition in registry for " + registryKey + " and  entity " + entityId;
                log.error(msg);
                return new JetspeedClearElement("JSR 168 Portlet is not configured correctly, no content available. Registry Key [" + registryKey + "], entity [" + entityId + "]");                
            }
            int mode = jrun.getMode();
            
            syncNavigationalState(window, jrun, mode, instance);
                                    
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        return service(data, entityId);
        // return dispatch(data, entityId);
    }
    
    public PortletWindow getPortletWindow(String id, String registryKey)
    {
        PortletWindow window = windowAccessor.getPortletWindow(id);
        if (window == null)
        {
            PortletEntity entity = entityAccess.getPortletEntity(id);
            if (entity == null)
            {
                PortletDefinition portletDef = registryAccess.getPortletDefinitionByUniqueName(registryKey);
                if (null == portletDef)
                {
                    // can't get the portlet either, give up
                    return null;
                }
                entity = entityAccess.newPortletEntityInstance(portletDef);
                window = windowAccessor.createPortletWindow(entity, id);
                try
                {
                    entityAccess.storePortletEntity(entity);                    
                }
                catch (Exception e)
                {
                    log.error("Failed to store portlet entity: " + registryKey, e);
                    return null;
                }               
            }   
            else
            {
                window = windowAccessor.createPortletWindow(entity, id);
            }
        }
        return window;
    }
    
    public ConcreteElement service(RunData data, String entityId) 
    {
        String result = "";
        
        RequestContextComponent contextComponent = null;
        RequestContext context = null;
        try
        {
            HttpServletRequest request = data.getRequest();
            HttpServletResponse response =  data.getResponse();
            ServletConfig config = data.getServletConfig();
            Engine engine = Jetspeed.getEngine();
            contextComponent = (RequestContextComponent)Jetspeed.getComponentManager().getComponent(RequestContextComponent.class);
            
            context = (RequestContext) data.getRequest().getAttribute(FUSION_NAV_STATE);
            if (context == null)
            {
                context = contextComponent.create(request, response, config);
                data.getRequest().setAttribute(JetspeedFusionPortlet.FUSION_NAV_STATE, context);                                
            }
            context.setAttribute(PortalReservedParameters.PORTLET_ENTITY, entityId);
            context.getRequest().removeAttribute(PortalReservedParameters.PIPELINE);
            
            engine.service(context);
            
        }
        catch (Throwable t)
        {
            t.printStackTrace();            
            return new JetspeedClearElement(t.getMessage());            
        }
        finally
        {
            if (contextComponent != null && context != null)
            {
                contextComponent.release(context);
            }                        
        }

        return new JetspeedClearElement(result);
    }
    
    /**
     * dispatch to fusion portlet
     * 
     * @param data
     */
    public ConcreteElement dispatch(RunData data, String entityId)
    {
        String result = "";
        
        try
        {
            HttpServletRequest request = data.getRequest();
            String dispatchURL = "/fusion/portal?pipeline=portlet-pipeline&entity=" + entityId;
            ServletContext fusion = data.getServletConfig().getServletContext().getContext("/fusion");
            if (fusion == null)
            {
                String message = "Dispatching ERROR: Failed to get Fusion webapp context";
                return new JetspeedClearElement(message);
            }
            
            RequestDispatcher jsDispatcher = fusion.getRequestDispatcher(dispatchURL);
            if (jsDispatcher == null)
            {
                String message = "Dispatching ERROR: Failed to get a request dispatcher for /jetspeed";
                return new JetspeedClearElement(message);
            }

            HttpServletResponse response =  data.getResponse(); 
            
            jsDispatcher.include(data.getRequest(), response);

            // response.getWriter().print("Hello JSR 168 out the print writer");
            
            // Note this is still under construction
            // result = "Portlet Content goes here";
        }
        catch (Throwable t)
        {
            t.printStackTrace();            
            return new JetspeedClearElement(t.getMessage());
        }

        return new JetspeedClearElement(result);        
    }

    
    public MutableNavigationalState getNavigationalState(JetspeedRunData data)
    {
        NavigationalState ns = null;
        RequestContext context = (RequestContext) data.getRequest().getAttribute(FUSION_NAV_STATE);
        if (context != null)
        {
            PortalURL url = context.getPortalURL();
            if (url != null)
                ns = url.getNavigationalState();
        }
        return (MutableNavigationalState)ns;
    }
    
    private void syncNavigationalState(PortletWindow window,
                                       JetspeedRunData data,
                                       int mode,
                                       PortletInstance instance)
    {
        MutableNavigationalState nav = getNavigationalState(data);
        RequestContext context = (RequestContext) data.getRequest().getAttribute(FUSION_NAV_STATE);        
        if (null == nav)
        {
            return;
        }
            
        PortletMode mode2 = PortletMode.VIEW;
        WindowState state = WindowState.NORMAL;
         
        mode2 = nav.getMode(window);
        state = nav.getState(window);
         
        if (isMinimized(data))
        {
            nav.setState(window, WindowState.MINIMIZED);
            nav.sync(context);
            return;            
        }
        
        
        String action = data.getAction();
        boolean targeted = isTargeted(data, instance, action);
         
        boolean syncRequired = false;
         
        if (!targeted)
        {
            if (mode2 != null && mode2.equals(PortletMode.VIEW))
            {
                
            }
        }
        else
        {            
            if (targeted && action.equals(J1_HELP_ACTION))
            {
                nav.setMode(window, PortletMode.HELP);
                nav.sync(context);            
                return;
            }
            if (targeted && action.equals(J1_RESTORE_ACTION))
            {
                nav.setMode(window, PortletMode.VIEW);
                nav.setState(window, WindowState.NORMAL);
                nav.sync(context);            
                return;
            }
            if (targeted && action.equals(J1_EDIT_ACTION))
            {
                nav.setMode(window, PortletMode.EDIT);
                nav.sync(context);            
                return;
            }
            
            switch (mode)
            {
            case JetspeedRunData.NORMAL:
                if (!mode2.equals(PortletMode.VIEW))
                {
                    nav.setMode(window, PortletMode.VIEW);
                    syncRequired = true;
                }
                if (!state.equals(WindowState.NORMAL))
                {
                    nav.setState(window, WindowState.NORMAL);
                    syncRequired = true;                
                }                
                break;
            case JetspeedRunData.MAXIMIZE:
                if (!state.equals(WindowState.MAXIMIZED))
                {
                    nav.setState(window, WindowState.MAXIMIZED);
                    syncRequired = true;                
                }                
                break;
            case JetspeedRunData.CUSTOMIZE:
                if (!mode2.equals(PortletMode.EDIT))
                {
                    nav.setMode(window, PortletMode.EDIT);
                    syncRequired = true;                
                }                
                break;                
            }
              
            if (syncRequired)
            {
                nav.sync(context);
            }
        }
    }
    
    public boolean providesCustomization()
    {
        return true;
    }
    
    public void setMinimized(boolean flag, RunData rundata)
    {
        super.setMinimized(flag, rundata);
        JetspeedRunData jrun = (JetspeedRunData)rundata;
        String entityId = null;
        
        PortletInstance instance = this.getInstance(jrun);
        entityId = instance.getAttribute(J2_ENTITY, "3");
        PortletWindow window = windowAccessor.getPortletWindow(entityId);
        if (window == null)
        {
            window = windowAccessor.createPortletWindow(entityId);
        }
        int mode = jrun.getMode();
        
        syncNavigationalState(window, jrun, mode, instance);        
    }

    private static final String SUPPORTS_EDIT_MODE = "org.apache.jetspeed.fusion.edit.mode";
    private static final String SUPPORTS_VIEW_MODE = "org.apache.jetspeed.fusion.view.mode";
    private static final String SUPPORTS_HELP_MODE = "org.apache.jetspeed.fusion.help.mode";
     
    private boolean supportsMode(PortletMode mode, RunData rundata, String modeType)
    {
        Boolean supportsMode = 
            (Boolean)PortletSessionState.getAttribute(this, rundata, modeType);
        if (supportsMode == null)
        {
            String pa = getPortletConfig().getInitParameter(PORTLET_APPLICATION_NAME, null);
            String pd = getPortletConfig().getInitParameter(PORTLET_DEFINITION_NAME, null);
            String registryKey = pa + "::" + pd;
            PortletDefinition pdef = registryAccess.getPortletDefinitionByUniqueName(registryKey);
            boolean supports = pdef.getContentTypeSet().supportsPortletMode(mode);
            PortletSessionState.setAttribute(this, rundata, modeType, new Boolean(supports));
            return supports;
        }
        else
        {
            return supportsMode.booleanValue();
        }        
    }
    
    public boolean getAllowEdit( RunData rundata )
    {
        return super.getAllowEdit(rundata) && supportsMode(PortletMode.EDIT, rundata, SUPPORTS_EDIT_MODE);
    }
 
    public boolean getAllowInfo( RunData rundata )
    {
        return supportsMode(PortletMode.HELP, rundata, SUPPORTS_HELP_MODE);
    }
     
    public boolean getAllowView( RunData rundata )
    {
        return super.getAllowView(rundata) && supportsMode(PortletMode.VIEW, rundata, SUPPORTS_VIEW_MODE);
    }
    
    public boolean getAllowMaximize( RunData rundata )
    {
        return super.getAllowEdit(rundata);
    }
        
    public boolean isTargeted(JetspeedRunData jdata, PortletInstance instance, String action)
    {
        String peid = jdata.getJs_peid();
        if (peid == null)
        {
            return false;
        }
        if (action == null || action.length() == 0)
        {
            return false;
        }
        return (peid.equals(instance.getEntry().getId()));        
    }
    
    /*    
    public boolean providesCustomization()
    {
        return true;
    }
    */
    
    /*
    public HttpServletResponse getResponseForWindow(HttpServletRequest req, HttpServletResponse resp)
    {
        PortletContent myContent = new PortletContent();

        synchronized (contents)
        {
            contents.put(window.getId(), myContent);
        }

        return new HttpBufferedResponse(request.getResponse(),myContent.getWriter());
    }
    */
    
}
