/*
 * Copyright 2000-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.portal.portlets.streaming;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;


/**
 * FusionResponseWrapper
 *
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @author <a href="mailto:serge.huber@jahia.org">Serge Huber</a>
 * @version $Id: FusionResponseWrapper.java,v 1.1 2004/06/26 01:23:40 taylor Exp $
 */
public class FusionResponseWrapper extends HttpServletResponseWrapper
{
    private FusionServletOutputStream outputStream = null;
    private PrintWriter printWriter = null;    
    private boolean isPassThrough = false;
    private String encoding = "ISO-8859-1";
    private String forceEncoding = null;
    private boolean outputStreamCalled = false;
    
    public FusionResponseWrapper(HttpServletResponse httpServletResponse,
                                 boolean isPassThrough, 
                                 String forceEncoding) 
    {
        super (httpServletResponse);
    }

    public FusionResponseWrapper(HttpServletResponse httpServletResponse)
    {
        super (httpServletResponse);
    }

    
    public PrintWriter getWriter()
    {
        checkStreams();
        return printWriter;
    }

    public ServletOutputStream getOutputStream() throws IOException
    {
        checkStreams();
        this.outputStreamCalled = true;        
        return outputStream;
    }

    private void checkStreams()
    {
        if ((outputStream == null) && (printWriter == null))
        {
            try
            {
                // printWriter = new StringPrintWriter(new StringWriter(),
                // outputStringBuffer);
                if (isPassThrough)
                {
                    outputStream = new FusionServletOutputStream(super
                            .getResponse().getOutputStream(), encoding);
                } else
                {
                    outputStream = new FusionServletOutputStream(encoding);
                }
                OutputStreamWriter streamWriter;
                if (encoding != null)
                {
                    streamWriter = new OutputStreamWriter(outputStream,
                            encoding);
                } else
                {
                    streamWriter = new OutputStreamWriter(outputStream);
                }
                printWriter = new PrintWriter(streamWriter, true);
            } catch (Exception e)
            {
                // logger.debug("Error creating StringPrintWriter object !", e);
            }
        }
    }
    
    public String getStringBuffer(boolean checkWriterError) throws IOException
    {
        try
        {
            if (outputStreamCalled)
            {
                // logger.debug("buffer=[" + strOutputStream.getBuffer(encoding)
                // + "]");
                return outputStream.getBuffer();
            } 
            else
            {
                if (outputStream == null)
                {
                    return null;
                } else
                {
                    if (checkWriterError && printWriter.checkError()) 
                    { 
                        throw new IOException(
                            "An error has occured while writing to output"); 
                    }
                    return outputStream.getBuffer();
                }
            }
        } 
        catch (Exception e)
        {
            return "";
        }
    }
    
    public String toString()
    {
        try
        {
            return getStringBuffer(true);
        }
        catch (Exception e)
        {
            return "Error getting portlet content.";
        }
    }
}
