/*
 * Copyright 2000-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.portal.portlets.streaming;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;

import javax.servlet.ServletOutputStream;


/**
 * FusionServletOutputStream
 *
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @author <a href="mailto:serge.huber@jahia.org">Serge Huber</a>
 * @version $Id: FusionServletOutputStream.java,v 1.1 2004/06/26 01:23:40 taylor Exp $
 */
public class FusionServletOutputStream extends ServletOutputStream
{
    private ByteArrayOutputStream byteArray = new ByteArrayOutputStream ();
    private OutputStreamWriter streamWriter;
    private ServletOutputStream existingStream;
    private OutputStreamWriter existingStreamWriter;
    private String encoding;

    public FusionServletOutputStream(String encoding) throws UnsupportedEncodingException 
    {
        super();
        this.encoding = encoding;
        streamWriter = new OutputStreamWriter (byteArray, encoding);
    }
    
    public FusionServletOutputStream (ServletOutputStream existingStream, String encoding)
    throws UnsupportedEncodingException 
    {
        super();
        this.existingStream = existingStream;
        this.encoding = encoding;
        streamWriter = new OutputStreamWriter (byteArray, encoding);
        existingStreamWriter = new OutputStreamWriter (existingStream, encoding);
    }
    
    
    public void write(char[] cbuf) throws IOException
    {
        streamWriter.write(cbuf, 0, cbuf.length);
        if (existingStream != null)
        {
            existingStreamWriter.write(cbuf, 0, cbuf.length);
        }
    }

    public void write(char[] cbuf, int off, int len) throws IOException
    {
        streamWriter.write(cbuf, off, len);
        if (existingStream != null)
        {
            existingStreamWriter.write(cbuf, off, len);
        }
    }

    public void write(int c) throws IOException
    {
        byteArray.write(c);
        if (existingStream != null)
        {
            existingStream.write(c);
        }
    }

    public void write(String s) throws IOException
    {
        streamWriter.write(s);
        if (existingStream != null)
        {
            existingStreamWriter.write(s);
        }
    }

    public void write(String str, int off, int len) throws IOException
    {
        streamWriter.write(str, off, len);
        if (existingStream != null)
        {
            existingStreamWriter.write(str, off, len);
        }
    }

    public String getBuffer() throws UnsupportedEncodingException
    {
        return byteArray.toString(encoding);
    }

    public void flush() throws IOException
    {
        if (existingStream != null)
        {
            existingStreamWriter.flush();
            existingStream.flush();
        }
    }

    public void close()
    {
    }

}
