/*
 * Copyright 2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.profiler.impl;

import org.apache.jetspeed.om.page.ContentPage;
import org.apache.jetspeed.pipeline.PipelineException;
import org.apache.jetspeed.pipeline.valve.AbstractValve;
import org.apache.jetspeed.pipeline.valve.ValveContext;
import org.apache.jetspeed.request.RequestContext;
import org.apache.jetspeed.services.rundata.JetspeedRunData;
import org.apache.jetspeed.services.rundata.JetspeedRunDataService;
import org.apache.turbine.services.TurbineServices;
import org.apache.turbine.services.rundata.RunDataService;

/**
 * Security Valve for use J2 and populating expected Principal and 
 * Subjects as expected by JSR 168 portlets from the J1 JetspeedUser
 *
 * @see TemplateSecureSessionValidator
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor</a>
 * @version $Id: FusionProfilerValveImpl.java,v 1.2 2005/05/23 23:57:46 taylor Exp $
 */
public class FusionProfilerValveImpl 
    extends AbstractValve 
{    
    private JetspeedRunDataService runDataService = null;

    /* (non-Javadoc)
     * @see org.apache.jetspeed.pipeline.valve.Valve#invoke(org.apache.jetspeed.request.RequestContext, org.apache.jetspeed.pipeline.valve.ValveContext)
     */
    public void invoke(RequestContext request, ValveContext context)
            throws PipelineException
    {        
        JetspeedRunData data = getRunDataService().getCurrentRunData();
        ContentPage page = new FusionPage(data.getProfile().getId()); 
        request.setPage(page);
        
        // Pass control to the next Valve in the Pipeline
        context.invokeNext(request);        
    }

    private JetspeedRunDataService getRunDataService()
    {
        if (runDataService == null)
        {
            runDataService =
                (JetspeedRunDataService) TurbineServices.getInstance().getService(RunDataService.SERVICE_NAME);
        }
        return runDataService;
    }
    
}
