/*
 * Copyright 2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.fusion.tools.pamanager;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.jetspeed.Jetspeed;
import org.apache.jetspeed.components.portletentity.PortletEntityAccessComponent;
import org.apache.jetspeed.components.portletregistry.PortletRegistry;
import org.apache.jetspeed.components.portletregistry.RegistryException;
import org.apache.jetspeed.container.window.PortletWindowAccessor;
import org.apache.jetspeed.factory.PortletFactory;
import org.apache.jetspeed.om.common.portlet.MutablePortletApplication;
import org.apache.jetspeed.om.profile.Entry;
import org.apache.jetspeed.om.profile.PSMLDocument;
import org.apache.jetspeed.om.profile.Profile;
import org.apache.jetspeed.om.profile.QueryLocator;
import org.apache.jetspeed.om.registry.Category;
import org.apache.jetspeed.om.registry.MediaTypeEntry;
import org.apache.jetspeed.om.registry.MediaTypeRegistry;
import org.apache.jetspeed.om.registry.PortletEntry;
import org.apache.jetspeed.om.registry.base.BaseCachedParameter;
import org.apache.jetspeed.services.PsmlManager;
import org.apache.jetspeed.services.Registry;
import org.apache.jetspeed.services.resources.FusionResources;
import org.apache.jetspeed.tools.pamanager.PortletApplicationManager;
import org.apache.jetspeed.util.descriptor.PortletApplicationWar;
import org.apache.pluto.om.common.Description;
import org.apache.pluto.om.common.DisplayName;
import org.apache.pluto.om.common.Language;
import org.apache.pluto.om.portlet.ContentType;
import org.apache.pluto.om.portlet.PortletDefinition;
import org.apache.turbine.util.TurbineConfig;

/**
 * FusionPAM
 *
 * @author <a href="mailto:jford@apache.org">Jeremy Ford</a>
 * @author <a href="mailto:taylor@apache.org">David Sean Taylor </a>
 * @author <a href="mailto:ate@douma.nu">Ate Douma</a>
 * @version $Id: FusionPAM.java,v 1.18 2005/05/24 00:12:16 taylor Exp $
 */
public class FusionPAM extends PortletApplicationManager
{    
    private static final Log log = LogFactory.getLog("deployment");
    
    private static String JSR168_PORTLET;
    private static String PORTLET_DEFINITION_NAME;
    private static String PORTLET_APPLICATION_NAME;
    
    
    private static final String JS1_REFERENCE_TYPE = "ref";
        
    public FusionPAM(PortletFactory portletFactory, PortletRegistry registry,
                                    PortletEntityAccessComponent entityAccess, PortletWindowAccessor windowAccess)
    {      
        super(portletFactory, registry, entityAccess, windowAccess);
        initJetspeedOne(null);        
    }
            
    public static void main(String args[])
    {
        System.out.println("*** Fusion PAM ***");
        
        initJetspeedOne(args);
        System.out.println("*** Exiting FUSION PAM ***");
    }
    
    protected static void initJetspeedOne(String args[])
    {
        //
        // initialize and bootstrap services
        //
        try
        {
        	if (args != null)
        	{
        		//String root =  "./stage/webapp"; // FOR TESTING only
        		String root = System.getProperty("pam.deploy.dir") + "/fusion";
        		log.info("Using PAM root: " + root);
        		String properties = "/WEB-INF/conf/FusionPAM.properties";
            
        		//String properties = "/WEB-INF/conf/TurbineResources.properties";
        		//System.out.println("Using PAM root: " + root);
            
        		TurbineConfig config = new TurbineConfig( root, properties);
        		config.initialize();
        		System.out.println("TURBINE CONFIG DONE Using PAM root: " + root);
        	}
            JSR168_PORTLET = FusionResources.getString(FusionResources.JSR168_PORTLET, "JSR168Portlet");
            PORTLET_DEFINITION_NAME = FusionResources.getString(FusionResources.PORTLET_DEFINITION_PARAMETER, "portlet_definition");
            PORTLET_APPLICATION_NAME = FusionResources.getString(FusionResources.PORTLET_APPLICATION_PARAMETER, "portlet_application");
            
            log.info("Using PDEF NAME " + PORTLET_DEFINITION_NAME);
            log.info("Using PAPP NAME " + PORTLET_APPLICATION_NAME);
            
        }
        catch (Exception e)
        {
            System.out.println("EXCEPTION IN TURBINE CONFIG");            
            
            String msg = "Fusion PAM: error initializing Turbine configuration";
            log.error(msg, e);
            System.out.println(msg);
            e.printStackTrace();
            System.exit(0);
        }

/*
        try
        {
            Enumeration entries = Registry.getNames();
            while (entries.hasMoreElements())
            {
                String name = (String)entries.nextElement();
                System.out.println("name = " + name);
            }
            
        }
        catch (org.apache.turbine.services.InstantiationException e)
        {
            String msg = "Fusion PAM: error loading Psml Exporter Service";
            log.error(msg, e);
            System.out.println(msg);
            e.printStackTrace();
            System.exit(0);
        }
*/                
    }
    
    protected MutablePortletApplication registerPortletApplication(PortletApplicationWar paWar,
                                                                   MutablePortletApplication oldPA, 
                                                                   boolean local,
                                                                   ClassLoader loader)
                    throws RegistryException
    {
        MutablePortletApplication app = super.registerPortletApplication(paWar, oldPA, local, loader);
        //create xreg        
        try
        {
            // remove entries from the registry
            // registry.processPortletApplicationTree(app, "remove");
            log.info("Saving the portlet.xml in the J1 registry...");
            
            Collection pds = app.getPortletDefinitions();
            Iterator pdsIter = pds.iterator();
            while (pdsIter.hasNext()) 
            {
                PortletDefinition pdef = (PortletDefinition) pdsIter.next();
                PortletEntry entry = createPortletEntry(app.getName(), pdef);
                                
                Registry.saveEntry(Registry.PORTLET, entry);
                System.out.println("Writing registry: " + entry.getName());
            }
            
        }        
        catch (Exception e1)
        {
            e1.printStackTrace();
            // store.getTransaction().rollback();
            log.error("Error processing rollback.  Attempting to rollback registry transaction.", e1);
        }
        return app;
    }
    
    protected void unregisterPortletApplication(MutablePortletApplication pa, boolean purgeEntityInfo)
                    throws RegistryException
    {
        try
        {
            // remove entries from the registry
            
            Collection pds = pa.getPortletDefinitions();
            Iterator pdsIter = pds.iterator();
            while (pdsIter.hasNext()) 
            {
                PortletDefinition pdef = (PortletDefinition) pdsIter.next();
                String name = pdef.getName();                
                Registry.removeEntry(Registry.PORTLET, name);                

                // remove PSML refs                
                if (purgeEntityInfo)
                {
                    removePSML(name);
                }
            }
            
        }        
        catch (Exception e1)
        {
            e1.printStackTrace();
            // store.getTransaction().rollback();
            log.error("Error processing rollback.  Attempting to rollback registry transaction.", e1);
        }
        super.unregisterPortletApplication(pa, purgeEntityInfo);
    }
    
    public static PortletEntry createPortletEntry(String paName, PortletDefinition portletDefinition)
    {
        //TODO:  do the lookup once, minor speed improvment
        //      validate parent entry exists
        PortletEntry parentEntry = (PortletEntry)Registry.getEntry(Registry.PORTLET, JSR168_PORTLET);
        
        //TODO:  speed improvement 2, only get categories once
        Iterator categoryIterator = parentEntry.listCategories();
        
        PortletEntry entry = (PortletEntry) Registry.createEntry(Registry.PORTLET);
        entry.setParent(JSR168_PORTLET);
        entry.setType(JS1_REFERENCE_TYPE);
        
        Description desc = portletDefinition.getDescription(Jetspeed.getDefaultLocale());
        if(desc != null && desc.getDescription() != null && desc.getDescription().length() > 0)
        {
            entry.setDescription(desc.getDescription());
        }
        
        DisplayName title = portletDefinition.getDisplayName(Jetspeed.getDefaultLocale());
        if (title != null && title.getDisplayName() != null && title.getDisplayName().length() > 0)
        {
            entry.setTitle(title.getDisplayName());
        }
        else
        {
            Language lang = portletDefinition.getLanguageSet().get(Jetspeed.getDefaultLocale());
            if (lang != null)
            {
                String tit = lang.getShortTitle();
                if(tit != null && tit.length() > 0)
                {
                    entry.setTitle(tit);
                    if (desc == null || desc.getDescription() == null || desc.getDescription().length() == 0)
                    {
                        String descTitle = lang.getTitle();
                        if (descTitle != null && descTitle.length() > 0)
                        {
                            entry.setDescription(descTitle);
                        }
                    }
                }
                else
                {
                    tit = lang.getTitle();
                    if (tit != null && tit.length() > 0)
                    {
                        entry.setTitle(tit);
                    }
                }
            }                
        }
        entry.setName(portletDefinition.getName());
        
        Iterator iter = portletDefinition.getContentTypeSet().iterator();
        while (iter.hasNext())
        {
            ContentType contentType = (ContentType) iter.next();
            String type = contentType.getContentType();
            
            log.info("Attempting to find media entry with mimetype " + type);
            
            //example:  content type text/html => media type html
            MediaTypeRegistry mediaRegistry = (MediaTypeRegistry) Registry.get(Registry.MEDIA_TYPE);
            Enumeration en = mediaRegistry.getEntries();
            while(en.hasMoreElements())
            {
                MediaTypeEntry mte = (MediaTypeEntry)en.nextElement();

                //TODO:  the type can have wildcards, will REGEXP handle this?
                /*
                RE r = null;
                RECompiler rc = null;
                
                try 
                {
                    rc = new RECompiler();
                    r = new RE();
                    r.setProgram(rc.compile(type));
                }
                catch (org.apache.regexp.RESyntaxException rex)
                {
                    log.warn("FusionPAM: error processing regular expression [" + type + "]: " + 
                             rex.toString());
                }
                */
                
                if(type.indexOf('*') != -1)
                {
                    log.info("Found mime type with wildcard.");
                }
                else if (type.equals(mte.getMimeType()))// || r.match(mte.getMimeType()))
                {
                    log.info("Found media entry " + mte.getName() + " that matches mimetype " + type);
                    entry.addMediaType(mte.getName());
                    break;
                }
            }
        }
        
        //TODO:  figure out security
        //entry.setSecurityRef()
        
        //TODO:  what parameters are needed for the JSR168 portlet?
        //NOTE:  we should not need to copy preferences to the parameters
        BaseCachedParameter pdefParam = new BaseCachedParameter();
        pdefParam.setName(PORTLET_DEFINITION_NAME);
        pdefParam.setValue(portletDefinition.getName());
        pdefParam.setHidden(true);
        
        entry.addParameter(pdefParam);
        
        BaseCachedParameter pappParam = new BaseCachedParameter();
        pappParam.setName(PORTLET_APPLICATION_NAME);
        pappParam.setValue(paName);
        pappParam.setHidden(true);
        
        entry.addParameter(pappParam);
        
        //copy categories
        while (categoryIterator.hasNext())
        {
            Category category = (Category) categoryIterator.next();
            
            //this will only add the category if it doesn't already exist
            //which, it shouldn't
            entry.addCategory(category.getName(), category.getGroup());
        }
        
        return entry;
    }

    /**
     * Removes all PSML entries in the entire portal for the given name
     * 
     * @param name of the portlet in the registry
     */
    private void removePSML(String name)
    {        
        QueryLocator locator = new QueryLocator(QueryLocator.QUERY_USER); // ALL        
        Iterator profiles = null;
        int count = 0;
        try
        {
            profiles = PsmlManager.query(locator);

            while (profiles.hasNext() )
            {
                Profile profile = (Profile)profiles.next();                
                PSMLDocument doc = profile.getDocument();
                // System.out.println("Scanning: " + doc.getName());
                Entry entry = doc.getEntry(name);
                if (entry != null)
                {
                    // System.out.println("***** REMOVING ENTRY : " + entry.getName());
                    doc.removeEntryById(entry.getId());
                    PsmlManager.store(profile);
                }
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
            log.error("Remove PSML: Failed to remove profiles for portlet: " + name, e );
        }        
    }
}
