/*
 * Copyright 2000-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.modules.actions.controllers;

import java.util.List;
import java.util.Map;

import javax.servlet.ServletRequest;

import org.apache.jetspeed.om.profile.psml.PsmlEntry;
import org.apache.jetspeed.portal.Portlet;
import org.apache.jetspeed.portal.PortletController;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.rundata.JetspeedRunData;
import org.apache.jetspeed.services.statemanager.SessionState;
import org.apache.jetspeed.util.template.JetspeedTool;
import org.apache.jetspeed.util.template.JspTemplate;
import org.apache.jetspeed.util.StringUtils;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

/**
 * XMultiColumnControllerAction
 *
 * @author <a href="mailto:junyang@cisco.com">Jun Yang</a>
 * @version $Id: XMultiColumnControllerAction.java,v 1.5 2004/08/28 19:28:55 jford Exp $
 */
public class XMultiColumnControllerAction extends MultiColumnControllerAction
{
	/**
     * Static initialization of the logger for this class
     */    
    private static final JetspeedLogger logger = JetspeedLogFactoryService.getLogger(XMultiColumnControllerAction.class.getName());
	
    protected void buildCustomizeContext(
        PortletController controller,
        Context context,
        RunData rundata
    )
    {
        super.buildCustomizeContext(controller, context, rundata);
        context.put("template", "x-multicolumn-customize.vm");
        context.put("action", "controllers.XMultiColumnControllerAction");

        context.put("includedContent", new JspTemplate(rundata, "/controllers/html/x-multicolumn-customize.jsp"));
        ServletRequest request = rundata.getRequest();
        request.setAttribute("jspContext", context);

        // debug
        Object[] columns = (Object[]) context.get("portlets");
        Map portletTitleMap = (Map) context.get("titles");
        for (int i = 0; i < columns.length; i++)
        {
            List portletList = (List) columns[i];
            for (int j = 0; j < portletList.size(); j++)
            {
                PsmlEntry entry = (PsmlEntry) portletList.get(j);
                String portletTitle = (String) portletTitleMap.get(entry.getId());
                String portletSkinName = entry.getSkin() == null ? "-- Default --" : entry.getSkin().getName();
                String portletSecurityId =
                    entry.getSecurityRef() == null ?
                    "-- Default --" :
                    entry.getSecurityRef().getParent();
                String controlListBox =
                (entry.getControl() != null && entry.getControl().getName() != null) ?
                    JetspeedTool.getPortletParameter(rundata, 
                                                     (Portlet) rundata.getUser().getTemp("customizer"),
                                                      "control", entry.getControl().getName()) :
                    JetspeedTool.getPortletParameter(rundata, (Portlet) rundata.getUser().getTemp("customizer"), "control");
                controlListBox = controlListBox.substring(12);
                controlListBox = controlListBox.replace('\n', ' ');
                controlListBox = controlListBox.replace('\r', ' ');
                int z = 0;
            }
        }
    }
    
    public void doSave(RunData data, Context context)
    {
        applyModelChanges(data, context);
        super.doSave(data, context);
    }

    public void doDelete(RunData data, Context context)
    {
        applyModelChanges(data, context);
        super.doDelete(data, context);
    }

    public void doControl(RunData data, Context context)
    {
        applyModelChanges(data, context);
        super.doControl(data, context);
    }

    protected void applyModelChanges(RunData data, Context context)
    {
        ServletRequest request = data.getRequest();
        String[] modelChangeList = request.getParameterValues("modelChangeList");
        if (modelChangeList != null)
        {
            String[] moves = StringUtils.stringToArray(modelChangeList[0], ";");

            if(moves != null)
            {
	            SessionState customizationState = ((JetspeedRunData) data).getPageSessionState();
	            List[] columns = (List[]) customizationState.getAttribute("customize-columns");
	
	            for (int i = 0; i < moves.length; i++)
	            {
	                String[] values = StringUtils.stringToArray(moves[i], ",");
	                if(values != null  && values.length >= 4)
	                {
		                int originCol = Integer.parseInt(values[0]);
		                int originRow = Integer.parseInt(values[1]);
		                int destCol = Integer.parseInt(values[2]);
		                int destRow = Integer.parseInt(values[3]);
		                
		                if(verifyColumnsBounds(originCol, destCol, columns))
		                {
		                	List originColumn = columns[originCol];
		                	if((originRow >= 0) && (originRow < originColumn.size()))
		                	{
		                	    Object portletEntry = originColumn.get(originRow);
		                		List destinationColumn = columns[destCol];
		                		
		                		if((destRow >=0) && (destRow < destinationColumn.size()))
		                		{
		                			originColumn.remove(originRow);
		                			destinationColumn.add(destRow, portletEntry);
		                		}
		                		else if(destRow >= destinationColumn.size())
		                		{
		                		    originColumn.remove(originRow);
		                		    destinationColumn.add(portletEntry);
		                		}
		                		else
		                		{
		                			logger.warn("Destination Row " + destRow + " is out of bounds.");
		                		}
		                	}
		                	else
		                	{
		                		logger.warn("Origin Row " + originRow + " is out of bounds.");
		                	}
		                }
	                }
	            }
            }
        }
    }
    
    private boolean verifyColumnsBounds(int originCol, int destCol, List[] columns)
    {
    	boolean result = false;
    	
    	result = (originCol >=0) && (originCol < columns.length);
    	if(result)
    	{
    		result = (destCol >=0) && (destCol < columns.length);
    		
    		if(!result)
    		{
    			logger.warn("Destination Column " + destCol + " is out of bounds.");
    		}
    	}
    	else
    	{
    		logger.warn("Origin Column " + originCol + " is out of bounds.");
    	}
    	
    	return result;
    }
}    

