/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jetspeed.modules.parameters;

//ecs stuff
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.util.StringUtils;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

/**
 * Returns simple list box control
 * <p>Options:
 * <UL>
 * <LI><code>items</code> - comma-separated list of list box items</LI>
 * <LI><code>layout</code> [<strong>$combo</strong>|$list] - combo box vs list box</LI> 
 * <LI><code>size</code> - size of the list box for $list style</LI> 
 * <LI><code>sort</code> [<strong>false</strong>|true] - return sorted list of items</LI>  
 * <LI><code>multiplechoice</code> [<strong>false</strong>|true] - allow multiple selections</LI>     
 * </UL> 
 * @author <a href="mark_orciuch@ngsltd.com">Mark Orciuch</a>
 * @version $Id: ListBox.java,v 1.6 2005/01/18 17:08:45 morciuch Exp $  
 */
public class ListBox
    extends org.apache.jetspeed.modules.parameters.VelocityParameterPresentationStyle
{
    private static final JetspeedLogger logger =
        JetspeedLogFactoryService.getLogger(ListBox.class.getName());

    public static final String ITEMS = "items";
    public static final String SORTED = "sorted";

    protected String items = null;
    protected boolean sorted = false;

    /**
     * Put custom objects in velocity context
     * 
     * @param data
     * @param name
     * @param value
     * @param parms
     * @param context
     * @return string
     */
    public void buildContext(
        RunData data,
        String name,
        String value,
        Map parms,
        Context context)
    {
        try
        {
            init(data);

            Map itemMap = getListItems(data, name, value, parms, context);

            context.put(ITEMS, itemMap);
        }
        catch (Exception e)
        {
            logger.error("Error building parameter context", e);
        }
    }

    /**
     * Returns a map of list items (key=value)
     * 
     * @return
     */
    protected Map getListItems(
        RunData data,
        String name,
        String value,
        Map parms,
        Context context)
    {
        if (items.indexOf("=") >= 0 && items.indexOf(",") >= 0)
        {
            return StringUtils.splitStringPairs(items, ",", "=", sorted);
        }
        else
        {
            Map result = new Hashtable();
            List list = StringUtils.splitString(items, ",");
            for (Iterator it = list.iterator(); it.hasNext();)
            {
                String item = (String) it.next();
                result.put(item, item);
            }
            return result;
        }
    }

    /**
     * Initialize options
     * 
     * @param data
     */
    protected void init(RunData data) throws Exception
    {
        sorted =
            new Boolean((String) this.getParm(SORTED, "false")).booleanValue();
        items = (String) this.getParm(ITEMS, null);
    }
}
