/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jetspeed.om.registry;

import java.util.Vector;

/**
 * Interface describing a parameter style for a portlet parameter. 
 *
 * @author <a href="mailto:mark_orciuch@ngsltd">Mark Orciuch</a>
 * @version $Id: ParameterStyle.java,v 1.1 2005/05/04 20:26:37 morciuch Exp $
 */
public interface ParameterStyle
{
	/**
	 * Returns parameter style name
	 * 
     * @return parameter style name
     */
    public String getName();
	
	/**
	 * Sets the parameter style name
	 * 
     * @param value
     */
    public void setName(String value);
	
	/**
	 * Returns optional parameter style template. If not
	 * specified it is assumed to be [parameter style name].vm or .jsp
	 * depending on the style name.
	 * 
     * @return parameter style template
     */
    public String getTemplate();
	
	/**
	 * Sets the optional parameter style template
	 * 
     * @param value
     */
    public void setTemplate(String value);
	
	/**
	 * Returns a vector of parameter style options
	 * 
     * @return parameter style options
     */
    public Vector getOptions();
	
	/**
	 * Returns specific style option
	 * 
     * @param name
     * @return option
     */
    public StyleOption getOption(String name);
	
	/**
	 * Adds a new parameter style option
	 * 
     * @param option to add
     */
    public void addOption(StyleOption option);
	
	/**
	 * Remove specific parameter style option
	 * 
     * @param name of style option
     */
    public void removeOption(String name);
	
	/**
	 * Sets parameter style options to a vector of options
	 * 
     * @param options
     */
    public void setOptions(Vector options);
	
	/**
	 * Returns true if specific style option exists
	 * 
     * @param name of the option
     * @return true or false
     */
    public boolean hasOption(String name);
}
