/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jetspeed.om.registry.base;

import java.util.Iterator;
import java.util.Vector;

import org.apache.jetspeed.om.registry.ParameterStyle;
import org.apache.jetspeed.om.registry.StyleOption;

/**
 * Bean like implementation of the ParameterStyle interface suitable for
 * Castor serialization.
 *
 * @see org.apache.jetspeed.om.registry.ParameterStyle
 * @author <a href="mailto:mark_orciuch@ngsltd.com">Mark Orciuch</a>
 * @version $Id: BaseParameterStyle.java,v 1.1 2005/05/04 20:26:37 morciuch Exp $
 */
public class BaseParameterStyle implements ParameterStyle
{
    private String name = null;
    private String template = null;
    private Vector options = new Vector();

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#getName()
     */
    public String getName()
    {
        return name;
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#setName(java.lang.String)
     */
    public void setName(String value)
    {
        name = value;
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#getTemplate()
     */
    public String getTemplate()
    {
        return template;
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#setTemplate(java.lang.String)
     */
    public void setTemplate(String value)
    {
        template = value;
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#getOptions()
     */
    public Vector getOptions()
    {
        return options;
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#getOption(java.lang.String)
     */
    public StyleOption getOption(String name)
    {
        Iterator it = options.iterator();
        while (it.hasNext())
        {
            StyleOption option = (StyleOption) it.next();
            if (option.getName().equals(name))
            {
                return option;
            }
        }

        return null;
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#addOption(org.apache.jetspeed.om.registry.StyleOption)
     */
    public void addOption(StyleOption option)
    {
    	options.add(option);
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#removeOption(java.lang.String)
     */
    public void removeOption(String name)
    {
		Iterator it = options.iterator();
		while (it.hasNext())
		{
			StyleOption option = (StyleOption) it.next();
			if (option.getName().equals(name))
			{
				options.remove(option);
			}
		}		
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#setOptions(java.util.Vector)
     */
    public void setOptions(Vector options)
    {
		this.options = options;
    }

    /* (non-Javadoc)
     * @see org.apache.jetspeed.om.registry.ParameterStyle#hasOption(java.lang.String)
     */
    public boolean hasOption(String name)
    {
		Iterator it = options.iterator();
		while (it.hasNext())
		{
			StyleOption option = (StyleOption) it.next();
			if (option.getName().equals(name))
			{
				return true;
			}
		}

        return false;
    }
}
