/*
 * Copyright 2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.services.jsp;

import java.io.PrintWriter;
import java.io.StringWriter;

import javax.servlet.RequestDispatcher;

import org.apache.turbine.services.jsp.TurbineJspService;
import org.apache.turbine.util.RunData;
import org.apache.turbine.util.TurbineException;

/**
 * This service is an extension on the TurbineJspService.  
 * It adds one method called handleBufferedRequest.
 * This method buffers the result of the JSP and returns it as a java.lang.String
 */
public class JetspeedJspService extends TurbineJspService 
{
    /**
     * Process the request
     *
     * @param RunData
     * @param String the filename of the template.
     * @param boolean whether to perform a forward or include.
     * @throws TurbineException Any exception trown while processing will be
     *         wrapped into a TurbineException and rethrown.
     * @return String
     */
    public String handleBufferedRequest(RunData data, String filename, boolean isForward)
        throws TurbineException
    {
    	String result = "";
    	
        /** template name with relative path */
        String relativeTemplateName = getRelativeTemplateName(filename);

        if (relativeTemplateName == null)
        {
            throw new TurbineException(
            "Template " + filename + " not found in template paths");
        }

        // get the RequestDispatcher for the JSP
        RequestDispatcher dispatcher = data.getServletContext()
        .getRequestDispatcher(relativeTemplateName);

        try
        {
        	StringWriter writer = new StringWriter();
        	PrintWriter printWriter = new PrintWriter(writer);
        	HttpBufferedResponse response = new HttpBufferedResponse(data.getResponse(), printWriter);
            if (isForward)
            {
                // forward the request to the JSP
                dispatcher.forward(data.getRequest(), response);
            }
            else
            {
                data.getOut().flush();
                // include the JSP
                dispatcher.include(data.getRequest(), response);
            }
            
            printWriter.flush();
            printWriter.close();
            
            result = writer.getBuffer().toString();
        }
        catch(Exception e)
        {
            /// pass the exception to the caller according to the general
            // contract for tamplating services in Turbine
            throw new TurbineException(
                "Error encountered processing a template:" + filename, e);
        }
        
        return result;
    }
}
