/*
 * Copyright 2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.services.messaging;

import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;

/**
 * This class provides an API for accessing a JMS component.
 * 
 * 
 * @author <a href="mailto:jford@apache.org">Jeremy Ford</a>
 * @version $Id: MessagingService.java,v 1.4 2004/08/25 01:16:53 jford Exp $
 */
public interface MessagingService
{
    public static final String SERVICE_NAME = "MessagingService";
    
    public static final String USER_UPDATE_SUBJECT = "org.apache.jetspeed.services.messaging.user.update";
    public static final String GROUP_UPDATE_SUBJECT = "org.apache.jetspeed.services.messaging.group.update";
    public static final String ROLE_UPDATE_SUBJECT = "org.apache.jetspeed.services.messaging.role.update";
    public static final String SECURITY_UPDATE_SUBJECT = "org.apache.jetspeed.services.messaging.security.update";
    
    public static final int MESSAGE = 0;
    public static final int TEXT_MESSAGE = 1;
    public static final int BYTE_MESSAGE = 2;
    public static final int MAP_MESSAGE = 3;
    public static final int OBJECT_MESSAGE = 4;
    public static final int STREAM_MESSAGE = 5;

    public static final String REGISTRY_UPDATE_SUBJECT = "org.apache.jetspeed.services.registry.update";

    public static final String SECURITY_CACHE_UPDATE_SUBJECT = "org.apache.jetspeed.services.security.cache.update";

    public static final String PSML_UPDATE_SUBJECT = "org.apache.jetspeed.services.psml.cache.update";;
    
    /**
     * This method creates a new consumer if one with the given id has not been created.
     * The consumer will be closed during the services shutdown.
     * 
     * @param listener
     * @param id
     * @param destination
     */
    public void addMessageListener(MessageListener listener, String id, String destination);
    
    /**
     * This method creates a new MessageConsumer.
     * It's lifecycle should be managed by the caller.
     * 
     * @param listener
     * @param destination
     */
    public MessageConsumer createConsumer(String destination);
    
    
    /**
     * Remove a message listener that was added to the service 
     * previously via the addMessageListener method.
     * 
     * @param id
     */
    public void removeMessageListener(String id);
    
    /**
     * Send a message to the chose destination
     * 
     * @param message
     * @param destination
     */
    public void sendMessage(Message message, String destination);
    /**
     * Create a message based on the give type
     * 
     * @param messageType
     * @return
     */
    public Message createMessage(int messageType);
}
