/*
 * Copyright 2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.services.messaging.jms.activemq;

import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.Session;

import org.apache.jetspeed.services.messaging.jms.AbstractJMSMessagingService;
import org.codehaus.activemq.ActiveMQConnection;
import org.codehaus.activemq.ActiveMQConnectionFactory;

/**
 * @author <a href="mailto:jford@apache.org">Jeremy Ford</a>
 * @version $Id: MessagingServiceImpl.java,v 1.1 2004/07/23 00:24:01 jford Exp $
 */
public class MessagingServiceImpl extends AbstractJMSMessagingService
{
    protected static final String DEFAULT_FACTORY = "JmsTopicConnectionFactory";
    
    protected static final String CONFIG_USER = "user";
    protected static final String CONFIG_PWD = "pwd";
    
    protected static final String DEFAULT_USER = ActiveMQConnection.DEFAULT_USER;
    protected static final String DEFAULT_PWD = ActiveMQConnection.DEFAULT_PASSWORD;
    
    private String user;
    private String pwd;

    
    public MessagingServiceImpl()
    {
        this.DEFAULT_URL = ActiveMQConnection.DEFAULT_URL;
        this.DEFAULT_ACK_MODE = Session.AUTO_ACKNOWLEDGE;
        this.DEFAULT_DELIVERY_MODE = DeliveryMode.NON_PERSISTENT;
        this.DEFAULT_DURABLE = false;
        this.DEFAULT_NOLOCAL = true;
        this.DEFAULT_TRANSACTED = false;
    }
    
    protected void initJMSConfiguration()
    {
        super.initJMSConfiguration();
        
        user = getConfiguration().getString(CONFIG_USER, DEFAULT_USER);
        pwd = getConfiguration().getString(CONFIG_PWD, DEFAULT_PWD);
    }
        
    protected void createConnection() throws JMSException {
        String providerURL = getProviderURL();
        ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(user, pwd, providerURL);
        connection = connectionFactory.createConnection();
    }
    
    protected void createSession() throws JMSException {
        session = connection.createSession(transacted, ackMode);
    }
}
