/*
 * Copyright 2000-2001,2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jetspeed.util;

import java.io.IOException;
import java.net.URL;

import org.exolab.castor.mapping.Mapping;
import org.exolab.castor.mapping.MappingException;
import org.xml.sax.InputSource;

/**
 * Thread safe wrapper for the castor Mapping class.<br>
 * This wrapper serializes all thread access to the loadMapping() methods.
 * 
 */
 public class SynchronizedMapping extends Mapping {

	private static final Object global_synch = new Object();


	/**
	 *  
	 */
	public SynchronizedMapping() 
	{
		super();
	}

	/**
	 * @param arg0
	 */
	public SynchronizedMapping(ClassLoader arg0) 
	{
		super(arg0);
	}


	public void loadMapping(InputSource input) throws MappingException, IOException 
	{
		synchronized (global_synch) 
		{
			super.loadMapping(input);
		}
	}

	public void loadMapping(String input) throws MappingException, IOException 
	{
		synchronized (global_synch) 
		{
			super.loadMapping(input);
		}
	}

	public void loadMapping(URL input) throws MappingException, IOException 
	{
		synchronized (global_synch) 
		{
			super.loadMapping(input);
		}
	}


}
